#!/bin/bash
#

# build_pandora.sh <product_directory> <e4|nu:e4> <debug|opt|prof> [tar] [source]

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e4|nu:e4> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
    qlist=`echo ${fullqual} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`
    qualver=`echo ${qlist} | tr " " "_"`

    if [ "${basequal}" = "e4" ]
    then
	shortqual=e
	cqual=e4
    elif [ "${basequal}" = "nu:e4" ] 
    then
	shortqual=nu
	cqual=e4
    elif [ "${basequal}" = "mu2e:e4" ] 
    then
	shortqual=mu2e
	cqual=e4
    else
       echo "ERROR: unrecognized qualifier"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

select_gcc()
{
    if [ "${cqual}" = "e4" ] ||  [ "${cqual}" = "gcc48" ]
    then
       gccdotver=4.8
       gccver=v4_8_1
    elif [ "${cqual}" = "gcc46" ]
    then
       gccdotver=4.6
       gccver=v4_6_1
    fi
}

check_gcc()
{
    gccshortver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
    gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
    if [ "${gccshortver}" = "${gccdotver}" ]
    then
      echo " OK - gcc version is ${gccfullver}"
    else
      echo "ERROR: wrong gcc version ${gccfullver}"
      exit 1
    fi
}

setup_gcc()
{
    setup gcc ${gccver}
    check_gcc
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual} -z ${product_dir}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual} -z ${product_dir}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=pandora
pkgver=v00_11b
ssibuildshims_version=v0_06_06
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/b//'`
pkgtarfile=Pandora-v00-11.tar.bz2

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

select_gcc

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

#pandora depends on root
if ups exist root v5_34_09 -q +nu:+e4:+${extraqual}
then
   echo "root v5_34_09 -q ${fullqual} exists"
else
     echo "ERROR: cannot find root v5_34_09 -q ${fullqual}"
     exit 1
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

declare_product

set -x

ups build pandora ${pkgver} -q ${fullqual} -z ${product_dir}

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: build failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi


# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "${package} is installed at ${PANDORA_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
