#!/bin/bash
#

# buildRoot.sh <product_directory> <e2|mu2e:e2|nu:e2|e4|mu2e:e4|nu:e4> <debug|opt|prof> [tar] [source]

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e2|mu2e:e2|nu:e2|e4|mu2e:e4|nu:e4> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
    qlist=`echo ${fullqual} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`
    qualver=`echo ${qlist} | tr " " "_"`

    if [ "${basequal}" = "e2" ] 
    then
	shortqual=e
	cqual=e2
    elif [ "${basequal}" = "nu:e2" ] 
    then
	shortqual=nu
	cqual=e2
    elif [ "${basequal}" = "mu2e:e2" ] 
    then
	shortqual=mu2e
	cqual=e2
    elif [ "${basequal}" = "e4" ] 
    then
	shortqual=e
	cqual=e4
    elif [ "${basequal}" = "nu:e4" ] 
    then
	shortqual=nu
	cqual=e4
    elif [ "${basequal}" = "mu2e:e4" ] 
    then
	shortqual=mu2e
	cqual=e4
    else
       echo "ERROR: unrecognized qualifier"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

select_gcc()
{
    if [ "${cqual}" = "e2" ] ||  [ "${cqual}" = "gcc47" ]
    then
       gccdotver=4.7
       gccver=v4_7_1
    elif [ "${cqual}" = "e4" ] ||  [ "${cqual}" = "gcc48" ]
    then
       gccdotver=4.8
       gccver=v4_8_1
    elif [ "${cqual}" = "gcc46" ]
    then
       gccdotver=4.6
       gccver=v4_6_1
    fi
}

check_gcc()
{
    gccshortver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
    gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
    if [ "${gccshortver}" = "${gccdotver}" ]
    then
      echo " OK - gcc version is ${gccfullver}"
    else
      echo "ERROR: wrong gcc version ${gccfullver}"
      exit 1
    fi
}

setup_gcc()
{
    setup gcc ${gccver}
    check_gcc
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual} -z ${product_dir}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual} -z ${product_dir}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=root
pkgver=v5_34_09
ssibuildshims_version=v0_06_03
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=root_v${pkgdotver}.source.tar.gz
patchfile=fermi_root_5_34.patch

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

select_gcc

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# use ROOTSYS/MyRules.mk
if [ "${extraqual}" = "opt" ]
then
    rules=MyRules_opt.mk
elif [ "${extraqual}" = "debug" ]
then
    extracommand="--build=debug"
    rules=MyRules_debug.mk
elif [ "${extraqual}" = "prof" ]
then
    rules=MyRules_prof.mk
fi

# declare now so we can setup
mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

declare_product

# setup root
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}

check_gcc

if [ -z ${FFTW_INC} ]
then
   echo "ERROR: failed to setup fftw"
   exit 1
else
   echo "FFTW_INC: ${FFTW_INC}"
fi

if [ -z ${LIBXML2_INC} ]
then
   echo "ERROR: failed to setup libxml"
   exit 1
else
   echo "LIBXML2_INC: ${LIBXML2_INC}"
fi

if [ "${shortqual}" = "nu" ] 
then
    if [ -z ${POSTGRESQL_VERSION} ]
    then
       echo "ERROR: failed to setup postgresql"
       exit 1
    else
       echo "postgresql version: ${POSTGRESQL_VERSION}"
    fi

    if [ -z ${MYSQL_CLIENT_VERSION} ]
    then
       echo "ERROR: failed to setup mysql_client"
       exit 1
    else
       echo "mysql_client version: ${MYSQL_CLIENT_VERSION}"
    fi
fi

if [ -z ${PYTHONHOME} ]
then
   echo "ERROR: PYTHONHOME is undefined"
   echo "       something is very wrong"
   exit 1
fi

cd ${sourcedir}
tar xzf ${tardir}/${pkgtarfile}

# get the extra rules 
if [ -n ${rules} ]
then
   cp ${patchdir}/${rules} ${blddir}/MyRules.mk
fi
if [ "${cqual}" = "e2" ] || [ "${cqual}" = "e4" ]
then
  c11="--enable-c++11"
  echo "CXXFLAGS += -std=c++11 -Wno-deprecated-declarations" >> ${blddir}/MyRules.mk
  echo "NOOPT += -std=c++11 -Wno-deprecated-declarations" >> ${blddir}/MyRules.mk
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}/root

# apply Fermi patches
patch -b -p0 < ${patchdir}/${patchfile}

cd ${blddir}

if [ "${shortqual}" = "nu" ]
then
${sourcedir}/root/configure ${extracommand} \
--with-cc=gcc \
--with-cxx=g++ \
--with-ld=g++ \
--enable-gdml ${c11} \
--enable-roofit \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-xrootd \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib \
--with-pythia6-libdir=${PYLIB} \
--with-pgsql-incdir=${POSTGRESQL_FQ_DIR}/include \
--with-pgsql-libdir=${POSTGRESQL_FQ_DIR}/lib \
--with-mysql-incdir=${MYSQL_CLIENT_FQ_DIR}/include \
--with-mysql-libdir=${MYSQL_CLIENT_FQ_DIR}/lib
elif [ "${shortqual}" = "mu2e" ]
then
${sourcedir}/root/configure ${extracommand} \
--with-cc=gcc \
--with-cxx=g++ \
--with-ld=g++ \
--enable-gdml ${c11} \
--enable-roofit \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-mysql \
--disable-xrootd \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib
else
${sourcedir}/root/configure ${extracommand} \
--with-cc=gcc \
--with-cxx=g++ \
--with-ld=g++ \
--enable-gdml ${c11} \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-mysql \
--disable-xrootd \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib
fi

make -j ${ncore}
# now install in ROOTSYS
make install 

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# need to fix paths:
# bin/root-config
# lib64/Geant4-9.5.2/Geant4*.cmake
# share/Geant4-9.5.2/geant4make/geant4make*sh

rm -f /tmp/root-fix-${qualdir}.sh

echo "#!/bin/bash" > /tmp/root-fix-${qualdir}.sh
##echo " set -x " >>  /tmp/root-fix-${qualdir}.sh
echo "mv ${pkgdir}/bin/root-config ${pkgdir}/bin/root-config.bak" >> /tmp/root-fix-${qualdir}.sh
echo "cat ${pkgdir}/bin/root-config.bak | sed 's%${LIBXML2_FQ_DIR}%\${LIBXML2_FQ_DIR}%g'  | sed 's%${FFTW_FQ_DIR}%\${FFTW_FQ_DIR}%g' > ${pkgdir}/bin/root-config" >> /tmp/root-fix-${qualdir}.sh
echo "chmod +x  ${pkgdir}/bin/root-config" >> /tmp/root-fix-${qualdir}.sh
chmod +x /tmp/root-fix-${qualdir}.sh
/tmp/root-fix-${qualdir}.sh

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "${package} is installed at ${ROOTSYS}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
